@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Harvesting Kit.
-# Copyright (C) 2013, 2014 CERN.
+# Copyright (C) 2013, 2014, 2015, 2017, 2019 CERN.
 #
 # Harvesting Kit is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@ import sys
 import time
 import requests
 import xml.dom.minidom
+import datetime
 
 from bs4 import BeautifulSoup
 from os import (listdir,
@@ -44,17 +45,19 @@ except ImportError:
     register_exception = lambda a, b: True
 
 try:
-    from invenio.config import CFG_TMPSHAREDDIR
+    from invenio.config import CFG_TMPSHAREDDIR, CFG_LOGDIR
 except ImportError:
     from distutils.sysconfig import get_python_lib
     CFG_TMPSHAREDDIR = join(get_python_lib(),
                             "harvestingkit",
                             "tmp")
+    CFG_LOGDIR = join(get_python_lib(),
+                      "harvestingkit",
+                      "log")
 
-from harvestingkit.utils import run_shell_command
+from harvestingkit.utils import (create_logger, make_user_agent, run_shell_command)
 
 from harvestingkit.scoap3utils import (
-    create_logger,
     MissingFFTError,
     extract_package as scoap3utils_extract_package
 )
@@ -63,18 +66,27 @@ from harvestingkit.minidom_utils import (get_value_in_tag,
                                          xml_to_text)
 from harvestingkit.config import CFG_DTDS_PATH as CFG_SCOAP3DTDS_PATH
 from harvestingkit.utils import (fix_journal_name,
-                                 create_record,
-                                 record_add_field,
-                                 record_xml_output,
                                  format_arxiv_id,
                                  add_nations_field,
                                  fix_dashes)
 
+from harvestingkit.bibrecord import (
+    record_add_field,
+    create_record,
+    record_xml_output,
+)
+
 CFG_ELSEVIER_ART501_PATH = join(CFG_SCOAP3DTDS_PATH, 'ja5_art501.zip')
 CFG_ELSEVIER_ART510_PATH = join(CFG_SCOAP3DTDS_PATH, 'ja5_art510.zip')
 CFG_ELSEVIER_ART520_PATH = join(CFG_SCOAP3DTDS_PATH, 'ja5_art520.zip')
+CFG_ELSEVIER_ART540_PATH = join(CFG_SCOAP3DTDS_PATH, 'ja5_art540.zip')
+CFG_ELSEVIER_ART550_PATH = join(CFG_SCOAP3DTDS_PATH, 'ja5_art550.zip')
+CFG_ELSEVIER_ART560_PATH = join(CFG_SCOAP3DTDS_PATH, 'ja5_art560.zip')
 CFG_ELSEVIER_SI510_PATH = join(CFG_SCOAP3DTDS_PATH, 'si510.zip')
 CFG_ELSEVIER_SI520_PATH = join(CFG_SCOAP3DTDS_PATH, 'si520.zip')
+CFG_ELSEVIER_SI540_PATH = join(CFG_SCOAP3DTDS_PATH, 'si540.zip')
+CFG_ELSEVIER_SI550_PATH = join(CFG_SCOAP3DTDS_PATH, 'si550.zip')
+CFG_ELSEVIER_SI560_PATH = join(CFG_SCOAP3DTDS_PATH, 'si560.zip')
 CFG_ELSEVIER_JID_MAP = {'PLB': 'Physics letters B',
                         'NUPHB': 'Nuclear Physics B',
                         'CEMGE': 'Chemical Geology',
@@ -107,11 +119,15 @@ class ElsevierPackage(object):
     def __init__(self, package_name=None, path=None,
                  run_locally=False, CONSYN=False,
                  journal_mappings={},
-                 extract_nations=False):
+                 extract_nations=False,
+                 no_harvest=False):
         self.CONSYN = CONSYN
         self.doi_package_name_mapping = []
         try:
-            self.logger = create_logger("Elsevier")
+            self.logger = create_logger(
+                "Elsevier",
+                filename=join(CFG_LOGDIR, 'scoap3_harvesting.log')
+            )
         except IOError:  # Could not access log file
                          # Use std.out for logging
             self.logger = self
@@ -122,25 +138,26 @@ class ElsevierPackage(object):
         if self.CONSYN:
             self.journal_mappings = journal_mappings
         else:
-            self.package_name = package_name
-            self.path = path
-            self.found_articles = []
-            self._found_issues = []
-            if run_locally:
-                from harvestingkit.contrast_out import ContrastOutConnector
-                self.conn = ContrastOutConnector(self.logger)
-                self.conn.run(run_locally)
-            else:
-                if not path and package_name:
-                    self.logger.info("Got package: %s" % (package_name,))
-                    self._extract_package()
-                elif not path and not package_name:
+            if not no_harvest:
+                self.package_name = package_name
+                self.path = path
+                self.found_articles = []
+                self._found_issues = []
+                if run_locally:
                     from harvestingkit.contrast_out import ContrastOutConnector
                     self.conn = ContrastOutConnector(self.logger)
-                    self.conn.run()
-            self._crawl_elsevier_and_find_main_xml()
-            self._crawl_elsevier_and_find_issue_xml()
-            self._build_doi_mapping()
+                    self.conn.run(run_locally)
+                else:
+                    if not path and package_name:
+                        self.logger.info("Got package: %s" % (package_name,))
+                        self._extract_package()
+                    elif not path and not package_name:
+                        from harvestingkit.contrast_out import ContrastOutConnector
+                        self.conn = ContrastOutConnector(self.logger)
+                        self.conn.run()
+                self._crawl_elsevier_and_find_main_xml()
+                self._crawl_elsevier_and_find_issue_xml()
+                self._build_doi_mapping()
         self.extract_nations = extract_nations
 
     def _extract_package(self):
@@ -205,6 +222,16 @@ class ElsevierPackage(object):
                               % (dirname, err))
             walk(self.path, visit, None)
 
+
+    def _extract_correct_dtd_package(self, si_name, path):
+        try:
+            ZipFile(eval("CFG_ELSEVIER_%s_PATH" % si_name.upper())).extractall(path)
+        except Exception as e:
+                raise e
+        for filename in listdir(join(path, si_name)):
+            rename(join(path, si_name, filename), join(path, filename))
+
+
     def _normalize_issue_dir_with_dtd(self, path):
         """
         issue.xml from Elsevier assume the existence of a local DTD.
@@ -214,17 +241,18 @@ class ElsevierPackage(object):
         """
         if exists(join(path, 'resolved_issue.xml')):
             return
-        if 'si510.dtd' in open(join(path, 'issue.xml')).read():
-            ZipFile(CFG_ELSEVIER_SI510_PATH).extractall(path)
-            for filename in listdir(join(path, 'si510')):
-                rename(join(path, 'si510', filename), join(path, filename))
-        elif 'si520.dtd' in open(join(path, 'issue.xml')).read():
-            ZipFile(CFG_ELSEVIER_SI520_PATH).extractall(path)
-            for filename in listdir(join(path, 'si520')):
-                rename(join(path, 'si520', filename), join(path, filename))
-        else:
+        issue_xml_content = open(join(path, 'issue.xml')).read()
+        sis = ['si510.dtd', 'si520.dtd', 'si540.dtd', 'si550.dtd', 'si560.dtd']
+        tmp_extracted = 0
+        for si in sis:
+            if si in issue_xml_content:
+                self._extract_correct_dtd_package(si.split('.')[0], path)
+                tmp_extracted = 1
+
+        if not tmp_extracted:
             message = "It looks like the path " + path
-            message += " does not contain an si510 or si520 issue.xml file"
+            message += " does not contain an %s in issue.xml file" % \
+                ', '.join([si.split('.')[0] for si in sis])
             self.logger.error(message)
             raise ValueError(message)
         command = ["xmllint", "--format", "--loaddtd",
@@ -246,17 +274,19 @@ class ElsevierPackage(object):
         """
         if exists(join(path, 'resolved_main.xml')):
             return
-        if 'art520' in open(join(path, 'main.xml')).read():
-            ZipFile(CFG_ELSEVIER_ART520_PATH).extractall(path)
-            for filename in listdir(join(path, 'art520')):
-                rename(join(path, 'art520', filename), join(path, filename))
-        elif 'art501' in open(join(path, 'main.xml')).read():
-            ZipFile(CFG_ELSEVIER_ART501_PATH).extractall(path)
-            for filename in listdir(join(path, 'art501')):
-                rename(join(path, 'art501', filename), join(path, filename))
-        else:
+        main_xml_content = open(join(path, 'main.xml')).read()
+        arts = ['art501.dtd', 'art510.dtd', 'art520.dtd', 'art540.dtd',
+                'art550.dtd', 'art560.dtd']
+        tmp_extracted = 0
+        for art in arts:
+            if art in main_xml_content:
+                self._extract_correct_dtd_package(art.split('.')[0], path)
+                tmp_extracted = 1
+
+        if not tmp_extracted:
             message = "It looks like the path " + path
-            message += "does not contain an si520 or si501 main.xml file"
+            message += "does not contain an %s in main.xml file" % \
+                ', '.join([art.split('.')[0] for art in arts])
             self.logger.error(message)
             raise ValueError(message)
         command = ["xmllint", "--format", "--loaddtd",
@@ -272,7 +302,7 @@ class ElsevierPackage(object):
     def _add_references(self, xml_doc, rec, refextract_callback=None):
         for label, authors, doi, issue, page, title, volume, year,\
                 textref, ext_link, isjournal, comment, journal, publisher,\
-                editors, book_title in self.get_references(xml_doc):
+                editors, book_title, links in self.get_references(xml_doc):
             subfields = []
             if textref and not authors:
                 textref = textref.replace('\"', '\'')
@@ -338,6 +368,9 @@ class ElsevierPackage(object):
                         subfields.append(('s', journal))
                 if textref:
                     subfields.append(('m', textref))
+                if links:
+                    for link in links:
+                        subfields.append(('u', link))
                 if subfields:
                     record_add_field(rec, '999', ind1='C', ind2='5',
                                      subfields=subfields)
@@ -399,21 +432,16 @@ class ElsevierPackage(object):
             print("Can't find abstract", file=sys.stderr)
 
     def get_keywords(self, xml_doc):
-        if self.CONSYN:
-            try:
-                head = xml_doc.getElementsByTagName("ja:head")[0]
-                keywords = head.getElementsByTagName("ce:keyword")
-                return [get_value_in_tag(keyword, "ce:text")
-                        for keyword in keywords]
-            except Exception:
-                print("Can't find keywords", file=sys.stderr)
+        head = xml_doc.getElementsByTagName("ja:head")
+        if not head: 
+            head = xml_doc.getElementsByTagName("cja:head")
+        if not head:
+            keywords = xml_doc.getElementsByTagName("ce:keyword")
         else:
-            try:
-                keywords = xml_doc.getElementsByTagName("ce:keyword")
-                return [get_value_in_tag(keyword, "ce:text")
-                        for keyword in keywords]
-            except Exception:
-                print("Can't find keywords", file=sys.stderr)
+            keywords = head[0].getElementsByTagName("ce:keyword")
+        return [get_value_in_tag(keyword, "ce:text") 
+                for keyword in keywords 
+                if get_value_in_tag(keyword, "ce:text")]
 
     def get_copyright(self, xml_doc):
         try:
@@ -465,7 +493,11 @@ class ElsevierPackage(object):
         if sa_affiliation:
             return xml_to_text(sa_affiliation[0], ', ')
         else:
-            return ""
+            affiliation = re.sub(r'^(\d+\ ?)',"",get_value_in_tag(affiliation, "ce:textfn"))
+            if affiliation:
+                return affiliation
+            else:
+                raise IndexError
 
     def _find_affiliations(self, xml_doc, doi):
         try:
@@ -475,11 +507,6 @@ class ElsevierPackage(object):
         except IndexError:
             message = "Elsevier paper: {0} is missing sa:affiliation."
             register_exception(alert_admin=True, prefix=message.format(doi))
-            return dict((aff.getAttribute("id").encode('utf-8'),
-                        re.sub(r'^(\d+\ ?)',
-                               "",
-                               get_value_in_tag(aff, "ce:textfn")))
-                        for aff in xml_doc.getElementsByTagName("ce:affiliation"))
 
     def _add_affiliations_to_author(self, author, affs):
         if affs:
@@ -557,7 +584,7 @@ class ElsevierPackage(object):
 
             return authors
 
-    def get_publication_information(self, xml_doc, path=''):
+    def get_publication_information(self, xml_doc, path='', timeout=60):
         if self.CONSYN:
             publication = get_value_in_tag(xml_doc, "prism:publicationName")
             doi = get_value_in_tag(xml_doc, "prism:doi")
@@ -565,6 +592,8 @@ class ElsevierPackage(object):
             issue = get_value_in_tag(xml_doc, "prism:number")
             first_page = get_value_in_tag(xml_doc, "prism:startingPage")
             last_page = get_value_in_tag(xml_doc, "prism:endingPage")
+            if not first_page:
+                first_page = get_value_in_tag(xml_doc, "bam:articleNumber")
             journal = publication.split(",")[0]
             journal, volume = fix_journal_name(journal, self.journal_mappings)
             try:
@@ -576,13 +605,14 @@ class ElsevierPackage(object):
             except IndexError:
                 pass
             vol = get_value_in_tag(xml_doc, "prism:volume")
-            if vol is "":
+            if vol is "" and path is not "":
                 # if volume is not present try to harvest it
                 try:
                     session = requests.session()
                     url = 'http://www.sciencedirect.com/science/article/pii'\
                           + path.split('/')[-1]
-                    r = session.get(url)
+                    headers = {'user-agent': make_user_agent()}
+                    r = session.get(url, headers=headers, timeout=timeout)
                     parsed_html = BeautifulSoup(r.text)
                     info = parsed_html.body.find(
                         'p', attrs={'class': 'volIssue'}).text.split()
@@ -596,25 +626,8 @@ class ElsevierPackage(object):
                     pass
             if vol:
                 volume += vol
-            try:
-                year = xml_doc.getElementsByTagName(
-                    'ce:copyright')[0].getAttribute("year")
-                year = year.encode('utf-8')
-            except IndexError:
-                year = ''
-            start_date = get_value_in_tag(xml_doc, "prism:coverDate")
-            if len(xml_doc.getElementsByTagName('ce:date-accepted')) > 0:
-                full_date = xml_doc.getElementsByTagName('ce:date-accepted')[0]
-                y = full_date.getAttribute('year').encode('utf-8')
-                m = full_date.getAttribute('month').encode('utf-8').zfill(2)
-                d = full_date.getAttribute('day').encode('utf-8').zfill(2)
-                start_date = "%s-%s-%s" % (y, m, d)
-            elif len(start_date) is 8:
-                start_date = time.strftime(
-                    '%Y-%m-%d', time.strptime(start_date, '%Y%m%d'))
-            elif len(start_date) is 6:
-                start_date = time.strftime(
-                    '%Y-%m', time.strptime(start_date, '%Y%m'))
+            start_date = self.get_publication_date(xml_doc)
+            year = start_date.split("-")[0]
             doi = get_value_in_tag(xml_doc, "ce:doi")
             return (journal, issn, volume, issue, first_page,
                     last_page, year, start_date, doi)
@@ -625,9 +638,46 @@ class ElsevierPackage(object):
             except KeyError:
                 return ('', '', '', '', '', '', '', '', doi)
 
+    def get_publication_date(self, xml_doc):
+        """Return the best effort start_date."""
+        start_date = get_value_in_tag(xml_doc, "prism:coverDate")
+        if not start_date:
+            start_date = get_value_in_tag(xml_doc, "prism:coverDisplayDate")
+            if not start_date:
+                start_date = get_value_in_tag(xml_doc, 'oa:openAccessEffective')
+                if start_date:
+                    start_date = datetime.datetime.strptime(
+                        start_date, "%Y-%m-%dT%H:%M:%SZ"
+                    )
+                    return start_date.strftime("%Y-%m-%d")
+            import dateutil.parser
+            #dateutil.parser.parse cant process dates like April-June 2016
+            start_date = re.sub('([A-Z][a-z]+)[\s\-][A-Z][a-z]+ (\d{4})', 
+                                r'\1 \2', start_date)
+            try:
+                date = dateutil.parser.parse(start_date)
+            except ValueError:
+                return ''
+            # Special case where we ignore the deduced day form dateutil
+            # in case it was not given in the first place.
+            if len(start_date.split(" ")) == 3:
+                return date.strftime("%Y-%m-%d")
+            else:
+                return date.strftime("%Y-%m")
+        else:
+            if len(start_date) is 8:
+                start_date = time.strftime(
+                    '%Y-%m-%d', time.strptime(start_date, '%Y%m%d'))
+            elif len(start_date) is 6:
+                start_date = time.strftime(
+                    '%Y-%m', time.strptime(start_date, '%Y%m'))
+            return start_date
+
     def _get_ref(self, ref, label):
         doi = get_value_in_tag(ref, "ce:doi")
         page = get_value_in_tag(ref, "sb:first-page")
+        if not page:
+            page = get_value_in_tag(ref, "sb:article-number")
         issue = get_value_in_tag(ref, "sb:issue")
         title = get_value_in_tag(ref, "sb:maintitle")
         volume = get_value_in_tag(ref, "sb:volume-nr")
@@ -639,7 +689,7 @@ class ElsevierPackage(object):
         textref = ref.getElementsByTagName("ce:textref")
         if textref:
             textref = xml_to_text(textref[0])
-        ext_link = format_arxiv_id(self.get_ref_link(ref, 'arxiv'), True)
+        ext_link = format_arxiv_id(self.get_ref_link(ref, 'arxiv'))
         authors = []
         for author in ref.getElementsByTagName("sb:author"):
             given_name = get_value_in_tag(author, "ce:given-name")
@@ -658,7 +708,9 @@ class ElsevierPackage(object):
         comment = get_value_in_tag(ref, "sb:comment")
         links = []
         for link in ref.getElementsByTagName("ce:inter-ref"):
-            links.append(xml_to_text(link))
+            linktext = xml_to_text(link)
+            if re.search('^https?:\/\/', linktext):
+                links.append(linktext)
         title = ""
         try:
             container = ref.getElementsByTagName("sb:contribution")[0]
@@ -730,7 +782,7 @@ class ElsevierPackage(object):
         year = re.sub(r'\D', '', year)
         return (label, authors, doi, issue, page, title, volume,
                 year, textref, ext_link, isjournal, comment, journal,
-                publisher, editors, book_title)
+                publisher, editors, book_title, links)
 
     def get_references(self, xml_doc):
         for ref in xml_doc.getElementsByTagName("ce:bib-reference"):
@@ -904,17 +956,16 @@ class ElsevierPackage(object):
                     keyword = xml_to_text(listitem)
                     if keyword not in keywords:
                         keywords.append(keyword)
-            if keywords:
-                for keyword in keywords:
-                    record_add_field(rec, '653', ind1='1',
-                                     subfields=[('a', keyword),
-                                                ('9', 'author')])
+            for keyword in keywords:
+                record_add_field(rec, '653', ind1='1',
+                                 subfields=[('a', keyword),
+                                            ('9', 'author')])
             journal, dummy = fix_journal_name(journal.strip(),
                                               self.journal_mappings)
             subfields = []
             doctype = self.get_doctype(xml_doc)
             try:
-                page_count = int(last_page) - int(first_page)
+                page_count = int(last_page) - int(first_page) + 1
                 record_add_field(rec, '300',
                                  subfields=[('a', str(page_count))])
             except ValueError:  # do nothing
