@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##
 ## This file is part of Harvesting Kit.
-## Copyright (C) 2013, 2014 CERN.
+## Copyright (C) 2013, 2014, 2015 CERN.
 ##
 ## Harvesting Kit is free software; you can redistribute it and/or
 ## modify it under the terms of the GNU General Public License as
@@ -31,10 +31,12 @@ except ImportError:
 from harvestingkit.minidom_utils import (get_value_in_tag,
                                          xml_to_text)
 from harvestingkit.utils import (format_arxiv_id,
-                                 record_add_field,
-                                 record_xml_output,
-                                 create_record,
                                  add_nations_field)
+from harvestingkit.bibrecord import (
+    record_add_field,
+    create_record,
+    record_xml_output,
+)
 from xml.dom.minidom import parse
 
 
@@ -175,8 +177,9 @@ class JATSParser(object):
         emails = {}
         for contact in xml.getElementsByTagName("corresp"):
             contact_id = contact.getAttribute("id").encode('utf-8')
-            text = xml_to_text(contact.getElementsByTagName('email')[0])
-            emails[contact_id] = text
+            if contact.getElementsByTagName('email'):
+                text = xml_to_text(contact.getElementsByTagName('email')[0])
+                emails[contact_id] = text
 
         implicit_affilations = True
         for author in authors:
