@@ -1,3 +1,25 @@
+# -*- coding: utf-8 -*-
+#
+# This file is part of Harvesting Kit.
+# Copyright (C) 2015 CERN.
+#
+# Harvesting Kit is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; either version 2 of the
+# License, or (at your option) any later version.
+#
+# Harvesting Kit is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Harvesting Kit; if not, write to the Free Software Foundation, Inc.,
+# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+
+
+"""Common data for tests."""
+
 journal_mappings = {'A&A': 'Astron.Astrophys.',
                     'A&amp;A': 'Astron.Astrophys.',
                     'ApJ': 'Astrophys.J.',
@@ -7,18 +29,11 @@ journal_mappings = {'A&A': 'Astron.Astrophys.',
                     'ApJS': 'Astrophys.J.Suppl.',
                     'BAAS': 'Bull.Am.Astron.Soc.',
                     'Physics Letters': 'Phys.Lett.'}
-
 edp_test_record = 'data/sample_edpsciences_record.xml'
 aps_test_record = 'data/sample_aps_record.xml'
-pos_test_record = 'data/sample_pos_record.xml'
-consyn_test_record = 'data/sample_consyn_record.xml'
 ws_test_record = 'data/sample_ws_record.xml'
 ws_erratum_test_record = 'data/sample_ws_erratum_record.xml'
 aps_output = 'data/sample_aps_output.xml'
 edp_output = 'data/sample_edp_output.xml'
-consyn_output = 'data/sample_consyn_output.xml'
 ws_output = 'data/sample_ws_output.xml'
 ws_erratum_output = 'data/sample_ws_erratum_output.xml'
-xmllint = 'data/sample_elsevier_issue_input.xml'
-xmllint_output = 'data/sample_elsevier_issue_output.xml'
-xmllint_resources = 'data/si520'
